import bisect
import sys
import blist
from blist.test import mapping_tests

def CmpToKey(mycmp):
    'Convert a cmp= function into a key= function'
    class K(object):
        def __init__(self, obj):
            self.obj = obj
        def __lt__(self, other):
            return mycmp(self.obj, other.obj) == -1
    return K

class sorteddict_test(mapping_tests.TestHashMappingProtocol):
    type2test = blist.sorteddict

    def _reference(self):
        """Return a dictionary of values which are invariant by storage
        in the object under test."""
        return {1:2, 3:4, 5:6}

    class Collider(object):
        def __init__(self, x):
            self.x = x
        def __repr__(self):
            return 'Collider(%r)' % self.x
        def __eq__(self, other):
            return self.__class__ == other.__class__ and self.x == other.x
        def __lt__(self, other):
            if type(self) != type(other):
                return NotImplemented
            return self.x < other.x
        def __hash__(self):
            return 42

    def test_repr(self):
        d = self._empty_mapping()
        self.assertEqual(repr(d), 'sorteddict({})')
        d[1] = 2
        self.assertEqual(repr(d), 'sorteddict({1: 2})')
        d = self._empty_mapping()
        d[1] = d
        self.assertEqual(repr(d), 'sorteddict({1: sorteddict({...})})')
        d = self._empty_mapping()
        d[self.Collider(1)] = 1
        d[self.Collider(2)] = 2
        self.assertEqual(repr(d),
                         'sorteddict({Collider(1): 1, Collider(2): 2})')
        d = self._empty_mapping()
        d[self.Collider(2)] = 2
        d[self.Collider(1)] = 1
        self.assertEqual(repr(d),
                         'sorteddict({Collider(1): 1, Collider(2): 2})')

        class Exc(Exception): pass

        class BadRepr(object):
            def __repr__(self):
                raise Exc()

        d = self._full_mapping({1: BadRepr()})
        self.assertRaises(Exc, repr, d)

    def test_mutatingiteration(self):
        pass

    def test_sort(self):
        u = self.type2test.fromkeys([1, 0])
        self.assertEqual(list(u.keys()), [0, 1])

        u = self.type2test.fromkeys([2,1,0,-1,-2])
        self.assertEqual(u, self.type2test.fromkeys([-2,-1,0,1,2]))
        self.assertEqual(list(u.keys()), [-2,-1,0,1,2])
        
        a = self.type2test.fromkeys(reversed(list(range(512))))
        self.assertEqual(list(a.keys()), list(range(512)))
        
        def revcmp(a, b): # pragma: no cover
            if a == b:
                return 0
            elif a < b:
                return 1
            else: # a > b
                return -1
        u = self.type2test.fromkeys([2,1,0,-1,-2], key=CmpToKey(revcmp))
        self.assertEqual(list(u.keys()), [2,1,0,-1,-2])
        
        # The following dumps core in unpatched Python 1.5:
        def myComparison(x,y):
           xmod, ymod = x%3, y%7
           if xmod == ymod:
               return 0
           elif xmod < ymod:
               return -1
           else: # xmod > ymod
               return 1

        self.type2test.fromkeys(list(range(12)), key=CmpToKey(myComparison))

        #def selfmodifyingComparison(x,y):
        #    z[x+y] = None
        #    return cmp(x, y)
        #z = self.type2test(CmpToKey(selfmodifyingComparison))
        #self.assertRaises(ValueError, z.update, [(i,i) for i in range(12)])
        
        self.assertRaises(TypeError, self.type2test.fromkeys, 42, 42, 42, 42)

    def test_view_indexing_without_key(self):
      self._test_view_indexing(key=None)

    def test_view_indexing_with_key(self):
      self._test_view_indexing(key=lambda x: -x)

    def _test_view_indexing(self, key):
      expected_items = [(3, "first"), (7, "second")]
      if key is not None:
        u = self.type2test(key, expected_items)
        expected_items.sort(key=lambda item: key(item[0]))
      else:
        u = self.type2test(expected_items)
        expected_items.sort()
      expected_keys, expected_values = list(zip(*expected_items))

      if sys.version_info[0] < 3:
        keys = u.viewkeys()
        values = u.viewvalues()
        items = u.viewitems()
      else:
        keys = u.keys()
        values = u.values()
        items = u.items()

      for i in range(len(expected_items)):
        self.assertEqual(keys[i], expected_keys[i])
        self.assertEqual(values[i], expected_values[i])
        self.assertEqual(items[i], expected_items[i])

      for i in range(-1, len(expected_items)+1):
        for j in range(-1, len(expected_items)+1):
          self.assertEqual(keys[i:j], blist.sortedset(expected_keys[i:j]))
          self.assertEqual(values[i:j], list(expected_values[i:j]))
          self.assertEqual(items[i:j], blist.sortedset(expected_items[i:j]))

      self.assertEqual(list(reversed(keys)), list(reversed(expected_keys)))
      for i, key in enumerate(expected_keys):
        self.assertEqual(keys.index(key), expected_keys.index(key))
        self.assertEqual(keys.count(key), 1)
        self.assertEqual(keys.bisect_left(key), i)
        self.assertEqual(keys.bisect_right(key), i+1)
      self.assertEqual(keys.count(object()), 0)
      self.assertRaises(ValueError, keys.index, object())

      for item in expected_items:
        self.assertEqual(items.index(item), expected_items.index(item))
        self.assertEqual(items.count(item), 1)
      self.assertEqual(items.count((7, "foo")), 0)
      self.assertEqual(items.count((object(), object())), 0)
      self.assertRaises(ValueError, items.index, (7, "foo"))
      self.assertRaises(ValueError, items.index, (object(), object()))



# This file based loosely on Python's list_tests.py.

import sys
import collections, operator
import gc
import random
import blist
from blist.test import unittest
from blist.test import list_tests, seq_tests

def CmpToKey(mycmp):
    'Convert a cmp= function into a key= function'
    class K(object):
        def __init__(self, obj):
            self.obj = obj
        def __lt__(self, other):
            return mycmp(self.obj, other.obj) == -1
    return K

class SortedBase(object):
    def build_items(self, n):
        return list(range(n))

    def build_item(self, x):
        return x

    def test_empty_repr(self):
        self.assertEqual('%s()' % self.type2test.__name__,
                         repr(self.type2test()))

    def validate_comparison(self, instance):
        if sys.version_info[0] < 3 and isinstance(instance, collections.Set):
            ops = ['ne', 'or', 'and', 'xor', 'sub']
        else:
            ops = ['lt', 'gt', 'le', 'ge', 'ne', 'or', 'and', 'xor', 'sub']
        operators = {}
        for op in ops:
            name = '__'+op+'__'
            operators['__'+op+'__'] = getattr(operator, name)

        class Other(object):
            def __init__(self):
                self.right_side = False
            def __eq__(self, other):
                self.right_side = True
                return True
            __lt__ = __eq__
            __gt__ = __eq__
            __le__ = __eq__
            __ge__ = __eq__
            __ne__ = __eq__
            __ror__ = __eq__
            __rand__ = __eq__
            __rxor__ = __eq__
            __rsub__ = __eq__

        for name, op in operators.items():
            if not hasattr(instance, name): continue
            other = Other()
            op(instance, other)
            self.assertTrue(other.right_side,'Right side not called for %s.%s'
                            % (type(instance), name))

    def test_right_side(self):
        self.validate_comparison(self.type2test())

    def test_delitem(self):
        items = self.build_items(2)
        a = self.type2test(items)
        del a[1]
        self.assertEqual(a, self.type2test(items[:1]))
        del a[0]
        self.assertEqual(a, self.type2test([]))

        a = self.type2test(items)
        del a[-2]
        self.assertEqual(a, self.type2test(items[1:]))
        del a[-1]
        self.assertEqual(a, self.type2test([]))

        a = self.type2test(items)
        self.assertRaises(IndexError, a.__delitem__, -3)
        self.assertRaises(IndexError, a.__delitem__, 2)

        a = self.type2test([])
        self.assertRaises(IndexError, a.__delitem__, 0)

        self.assertRaises(TypeError, a.__delitem__)

    def test_delslice(self):
        items = self.build_items(2)
        a = self.type2test(items)
        del a[1:2]
        del a[0:1]
        self.assertEqual(a, self.type2test([]))

        a = self.type2test(items)
        del a[1:2]
        del a[0:1]
        self.assertEqual(a, self.type2test([]))

        a = self.type2test(items)
        del a[-2:-1]
        self.assertEqual(a, self.type2test(items[1:]))

        a = self.type2test(items)
        del a[-2:-1]
        self.assertEqual(a, self.type2test(items[1:]))

        a = self.type2test(items)
        del a[1:]
        del a[:1]
        self.assertEqual(a, self.type2test([]))

        a = self.type2test(items)
        del a[1:]
        del a[:1]
        self.assertEqual(a, self.type2test([]))

        a = self.type2test(items)
        del a[-1:]
        self.assertEqual(a, self.type2test(items[:1]))

        a = self.type2test(items)
        del a[-1:]
        self.assertEqual(a, self.type2test(items[:1]))

        a = self.type2test(items)
        del a[:]
        self.assertEqual(a, self.type2test([]))

    def test_out_of_range(self):
        u = self.type2test()
        def del_test():
            del u[0]
        self.assertRaises(IndexError, lambda: u[0])
        self.assertRaises(IndexError, del_test)

    def test_bad_mul(self):
        u = self.type2test()
        self.assertRaises(TypeError, lambda: u * 'q')
        def imul_test():
            u = self.type2test()
            u *= 'q'
        self.assertRaises(TypeError, imul_test)

    def test_pop(self):
        lst = self.build_items(20)
        random.shuffle(lst)
        u = self.type2test(lst)
        for i in range(20-1,-1,-1):
            x = u.pop(i)
            self.assertEqual(x, i)
        self.assertEqual(0, len(u))

    def test_reversed(self):
        lst = list(range(20))
        a = self.type2test(lst)
        r = reversed(a)
        self.assertEqual(list(r), list(range(19, -1, -1)))
        if hasattr(r, '__next__'): # pragma: no cover
            self.assertRaises(StopIteration, r.__next__)
        else: # pragma: no cover
            self.assertRaises(StopIteration, r.next)
        self.assertEqual(self.type2test(reversed(self.type2test())),
                         self.type2test())

    def test_mismatched_types(self):
        class NotComparable:
            def __lt__(self, other): # pragma: no cover
                raise TypeError
            def __cmp__(self, other): # pragma: no cover
                raise TypeError
        NotComparable = NotComparable()

        item = self.build_item(5)
        sl = self.type2test()
        sl.add(item)
        self.assertRaises(TypeError, sl.add, NotComparable)
        self.assertFalse(NotComparable in sl)
        self.assertEqual(sl.count(NotComparable), 0)
        sl.discard(NotComparable)
        self.assertRaises(ValueError, sl.index, NotComparable)

    def test_order(self):
        stuff = [self.build_item(random.randrange(1000000))
                 for i in range(1000)]
        if issubclass(self.type2test, collections.Set):
            stuff = set(stuff)
        sorted_stuff = list(sorted(stuff))
        u = self.type2test

        self.assertEqual(sorted_stuff, list(u(stuff)))
        sl = u()
        for x in stuff:
            sl.add(x)
        self.assertEqual(sorted_stuff, list(sl))
        x = sorted_stuff.pop(len(stuff)//2)
        sl.discard(x)
        self.assertEqual(sorted_stuff, list(sl))

    def test_constructors(self):
        # Based on the seq_test, but without adding incomparable types
        # to the list.

        l0 = self.build_items(0)
        l1 = self.build_items(1)
        l2 = self.build_items(2)

        u = self.type2test()
        u0 = self.type2test(l0)
        u1 = self.type2test(l1)
        u2 = self.type2test(l2)

        uu = self.type2test(u)
        uu0 = self.type2test(u0)
        uu1 = self.type2test(u1)
        uu2 = self.type2test(u2)

        v = self.type2test(tuple(u))
        class OtherSeq:
            def __init__(self, initseq):
                self.__data = initseq
            def __len__(self):
                return len(self.__data)
            def __getitem__(self, i):
                return self.__data[i]
        s = OtherSeq(u0)
        v0 = self.type2test(s)
        self.assertEqual(len(v0), len(s))

    def test_sort(self):
        # based on list_tests.py
        lst = [1, 0]
        lst = [self.build_item(x) for x in lst]
        u = self.type2test(lst)
        self.assertEqual(list(u), [0, 1])

        lst = [2,1,0,-1,-2]
        lst = [self.build_item(x) for x in lst]
        u = self.type2test(lst)
        self.assertEqual(list(u), [-2,-1,0,1,2])

        lst = list(range(512))
        lst = [self.build_item(x) for x in lst]
        a = self.type2test(reversed(lst))
        self.assertEqual(list(a), lst)

        def revcmp(a, b): # pragma: no cover
            if a == b:
                return 0
            elif a < b:
                return 1
            else: # a > b
                return -1
        u = self.type2test(u, key=CmpToKey(revcmp))
        self.assertEqual(list(u), [2,1,0,-1,-2])

        # The following dumps core in unpatched Python 1.5:
        def myComparison(x,y):
           xmod, ymod = x%3, y%7
           if xmod == ymod:
               return 0
           elif xmod < ymod:
               return -1
           else: # xmod > ymod
               return 1
        z = self.type2test(list(range(12)), key=CmpToKey(myComparison))

        self.assertRaises(TypeError, self.type2test, 42, 42, 42, 42)

class StrongSortedBase(SortedBase, seq_tests.CommonTest):
    def not_applicable(self):
        pass
    test_repeat = not_applicable
    test_imul = not_applicable
    test_addmul = not_applicable
    test_iadd = not_applicable
    test_getslice = not_applicable
    test_contains_order = not_applicable
    test_contains_fake = not_applicable

    def test_constructors2(self):
        s = "a seq"
        vv = self.type2test(s)
        self.assertEqual(len(vv), len(s))

        # Create from various iteratables
        for s in ("123", "", list(range(1000)), (1.5, 1.2), range(2000,2200,5)):
            for g in (seq_tests.Sequence, seq_tests.IterFunc,
                      seq_tests.IterGen, seq_tests.itermulti,
                      seq_tests.iterfunc):
                self.assertEqual(self.type2test(g(s)), self.type2test(s))
            self.assertEqual(self.type2test(seq_tests.IterFuncStop(s)),
                             self.type2test())
            self.assertEqual(self.type2test(c for c in "123"),
                             self.type2test("123"))
            self.assertRaises(TypeError, self.type2test,
                              seq_tests.IterNextOnly(s))
            self.assertRaises(TypeError, self.type2test,
                              seq_tests.IterNoNext(s))
            self.assertRaises(ZeroDivisionError, self.type2test,
                              seq_tests.IterGenExc(s))

class weak_int:
    def __init__(self, v):
        self.value = v
    def unwrap(self, other):
        if isinstance(other, weak_int):
            return other.value
        return other
    def __hash__(self):
        return hash(self.value)
    def __repr__(self): # pragma: no cover
        return repr(self.value)
    def __lt__(self, other):
        return self.value < self.unwrap(other)
    def __le__(self, other):
        return self.value <= self.unwrap(other)
    def __gt__(self, other):
        return self.value > self.unwrap(other)
    def __ge__(self, other):
        return self.value >= self.unwrap(other)
    def __eq__(self, other):
        return self.value == self.unwrap(other)
    def __ne__(self, other):
        return self.value != self.unwrap(other)
    def __mod__(self, other):
        return self.value % self.unwrap(other)
    def __neg__(self):
        return weak_int(-self.value)

class weak_manager():
    def __init__(self):
        self.all = [weak_int(i) for i in range(10)]
        self.live = [v for v in self.all if random.randrange(2)]
        random.shuffle(self.all)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        del self.all
        gc.collect()

class WeakSortedBase(SortedBase, unittest.TestCase):
    def build_items(self, n):
        return [weak_int(i) for i in range(n)]

    def build_item(self, x):
        return weak_int(x)

    def test_collapse(self):
        items = self.build_items(10)
        u = self.type2test(items)
        del items
        gc.collect()
        self.assertEqual(list(u), [])

    def test_sort(self):
        # based on list_tests.py
        x = [weak_int(i) for i in [1, 0]]
        u = self.type2test(x)
        self.assertEqual(list(u), list(reversed(x)))

        x = [weak_int(i) for i in [2,1,0,-1,-2]]
        u = self.type2test(x)
        self.assertEqual(list(u), list(reversed(x)))

        #y = [weak_int(i) for i in reversed(list(range(512)))]
        #a = self.type2test(y)
        #self.assertEqual(list(a), list(reversed(y)))

        def revcmp(a, b): # pragma: no cover
            if a == b:
                return 0
            elif a < b:
                return 1
            else: # a > b
                return -1
        u = self.type2test(u, key=CmpToKey(revcmp))
        self.assertEqual(list(u), x)

        # The following dumps core in unpatched Python 1.5:
        def myComparison(x,y):
           xmod, ymod = x%3, y%7
           if xmod == ymod:
               return 0
           elif xmod < ymod:
               return -1
           else: # xmod > ymod
               return 1
        x = [weak_int(i) for i in range(12)]
        z = self.type2test(x, key=CmpToKey(myComparison))

        self.assertRaises(TypeError, self.type2test, 42, 42, 42, 42)

    def test_constructor(self):
        with weak_manager() as m:
            wsl = self.type2test(m.all)
        self.assertEqual(list(wsl), m.live)

    def test_add(self):
        with weak_manager() as m:
            wsl = self.type2test()
            for x in m.all:
                wsl.add(x)
            del x
        self.assertEqual(list(wsl), m.live)

    def test_discard(self):
        with weak_manager() as m:
            wsl = self.type2test(m.all)
        x = m.live.pop(len(m.live)//2)
        wsl.discard(x)
        self.assertEqual(list(wsl), m.live)

    def test_contains(self):
        with weak_manager() as m:
            wsl = self.type2test(m.all)
        for x in m.live:
            self.assertTrue(x in wsl)
        self.assertFalse(weak_int(-1) in wsl)

    def test_iter(self):
        with weak_manager() as m:
            wsl = self.type2test(m.all)
        for i, x in enumerate(wsl):
            self.assertEqual(x, m.live[i])

    def test_getitem(self):
        with weak_manager() as m:
            wsl = self.type2test(m.all)
        for i in range(len(m.live)):
            self.assertEqual(wsl[i], m.live[i])

    def test_reversed(self):
        with weak_manager() as m:
            wsl = self.type2test(m.all)
        r1 = list(reversed(wsl))
        r2 = list(reversed(m.live))
        self.assertEqual(r1, r2)

        all = [weak_int(i) for i in range(6)]
        wsl = self.type2test(all)
        del all[-1]
        self.assertEqual(list(reversed(wsl)), list(reversed(all)))

    def test_index(self):
        with weak_manager() as m:
            wsl = self.type2test(m.all)
        for x in m.live:
            self.assertEqual(wsl[wsl.index(x)], x)
        self.assertRaises(ValueError, wsl.index, weak_int(-1))

    def test_count(self):
        with weak_manager() as m:
            wsl = self.type2test(m.all)
        for x in m.live:
            self.assertEqual(wsl.count(x), 1)
        self.assertEqual(wsl.count(weak_int(-1)), 0)

    def test_getslice(self):
        with weak_manager() as m:
            wsl = self.type2test(m.all)
        self.assertEqual(m.live, list(wsl[:]))

class SortedListMixin:
    def test_eq(self):
        items = self.build_items(20)
        u = self.type2test(items)
        v = self.type2test(items, key=lambda x: -x)
        self.assertNotEqual(u, v)

    def test_cmp(self):
        items = self.build_items(20)
        u = self.type2test(items)
        low = u[:10]
        high = u[10:]
        self.assert_(low != high)
        self.assert_(low == u[:10])
        self.assert_(low < high)
        self.assert_(low <= high, str((low, high)))
        self.assert_(high > low)
        self.assert_(high >= low)
        self.assertFalse(low == high)
        self.assertFalse(high < low)
        self.assertFalse(high <= low)
        self.assertFalse(low > high)
        self.assertFalse(low >= high)

        low = u[:5]
        self.assert_(low != high)
        self.assertFalse(low == high)

    def test_update(self):
        items = self.build_items(20)
        u = self.type2test()
        u.update(items)
        self.assertEqual(u, self.type2test(items))

    def test_remove(self):
        items = self.build_items(20)
        u = self.type2test(items)
        u.remove(items[-1])
        self.assertEqual(u, self.type2test(items[:19]))
        self.assertRaises(ValueError, u.remove, items[-1])

    def test_mul(self):
        items = self.build_items(2)
        u1 = self.type2test(items[:1])
        u2 = self.type2test(items)
        self.assertEqual(self.type2test(), u2*0)
        self.assertEqual(self.type2test(), 0*u2)
        self.assertEqual(self.type2test(), u2*0)
        self.assertEqual(self.type2test(), 0*u2)
        self.assertEqual(u2, u2*1)
        self.assertEqual(u2, 1*u2)
        self.assertEqual(u2, u2*1)
        self.assertEqual(u2, 1*u2)
        self.assertEqual(self.type2test(items + items), u2*2)
        self.assertEqual(self.type2test(items + items), 2*u2)
        self.assertEqual(self.type2test(items + items + items), 3*u2)
        self.assertEqual(self.type2test(items + items + items), u2*3)

        class subclass(self.type2test):
            pass
        u3 = subclass(items)
        self.assertEqual(u3, u3*1)
        self.assert_(u3 is not u3*1)

    def test_imul(self):
        items = self.build_items(2)
        items6 = items[:1]*3 + items[1:]*3
        u = self.type2test(items)
        u *= 3
        self.assertEqual(u, self.type2test(items6))
        u *= 0
        self.assertEqual(u, self.type2test([]))
        s = self.type2test([])
        oldid = id(s)
        s *= 10
        self.assertEqual(id(s), oldid)

    def test_repr(self):
        name = self.type2test.__name__
        u = self.type2test()
        self.assertEqual(repr(u), '%s()' % name)
        items = self.build_items(3)
        u.update(items)
        self.assertEqual(repr(u), '%s([0, 1, 2])' % name)
        u = self.type2test()
        u.update([u])
        self.assertEqual(repr(u), '%s([%s(...)])' % (name, name))

    def test_bisect(self):
        items = self.build_items(5)
        del items[0]
        del items[2] # We end up with [1, 2, 4]
        u = self.type2test(items, key=lambda x: -x) # We end up with [4, 2, 1]
        self.assertEqual(u.bisect_left(3), 1)
        self.assertEqual(u.bisect(2), 2) # bisect == bisect_right
        self.assertEqual(u.bisect_right(2), 2)

class SortedSetMixin:
    def test_duplicates(self):
        u = self.type2test
        ss = u()
        stuff = [weak_int(random.randrange(100000)) for i in range(10)]
        sorted_stuff = list(sorted(stuff))
        for x in stuff:
            ss.add(x)
        for x in stuff:
            ss.add(x)
        self.assertEqual(sorted_stuff, list(ss))
        x = sorted_stuff.pop(len(stuff)//2)
        ss.discard(x)
        self.assertEqual(sorted_stuff, list(ss))

    def test_eq(self):
        items = self.build_items(20)
        u = self.type2test(items)
        v = self.type2test(items, key=lambda x: -x)
        self.assertEqual(u, v)

    def test_remove(self):
        items = self.build_items(20)
        u = self.type2test(items)
        u.remove(items[-1])
        self.assertEqual(u, self.type2test(items[:19]))
        self.assertRaises(KeyError, u.remove, items[-1])

class SortedListTest(StrongSortedBase, SortedListMixin):
    type2test = blist.sortedlist

class WeakSortedListTest(WeakSortedBase, SortedListMixin):
    type2test = blist.weaksortedlist

    def test_advance(self):
        items = [weak_int(0), weak_int(0)]
        u = self.type2test(items)
        del items[0]
        gc.collect()
        self.assertEqual(u.count(items[0]), 1)

class SortedSetTest(StrongSortedBase, SortedSetMixin):
    type2test = blist.sortedset

class WeakSortedSetTest(WeakSortedBase, SortedSetMixin):
    type2test = blist.weaksortedset

    def test_repr(self):
        items = self.build_items(20)
        u = self.type2test(items)
        self.assertEqual(repr(u), 'weaksortedset(%s)' % repr(items))



#!/usr/bin/python
from __future__ import print_function

"""
Copyright 2007-2010 Stutzbach Enterprises, LLC (daniel@stutzbachenterprises.com)

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

   1. Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
   2. Redistributions in binary form must reproduce the above
      copyright notice, this list of conditions and the following
      disclaimer in the documentation and/or other materials provided
      with the distribution.
   3. The name of the author may not be used to endorse or promote
      products derived from this software without specific prior written
      permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
POSSIBILITY OF SUCH DAMAGE.

"""


import sys
import os

import unittest, operator
import blist, pickle
from blist import _blist
#BList = list
from blist.test import test_support, list_tests, sortedlist_tests, btuple_tests
from blist.test import sorteddict_tests, test_set

limit = _blist._limit
n = 512//8 * limit

class BListTest(list_tests.CommonTest):
    type2test = blist.blist

    def test_delmul(self):
        x = self.type2test(list(range(10000)))
        for i in range(100):
            del x[len(x)//4:3*len(x)//4]
            x *= 2

    def test_truth(self):
        super(BListTest, self).test_truth()
        self.assert_(not self.type2test())
        self.assert_(self.type2test([42]))

    def test_identity(self):
        self.assert_(self.type2test([]) is not self.type2test([]))

    def test_len(self):
        super(BListTest, self).test_len()
        self.assertEqual(len(self.type2test()), 0)
        self.assertEqual(len(self.type2test([0])), 1)
        self.assertEqual(len(self.type2test([0, 1, 2])), 3)

    def test_append2(self):
        lst = self.type2test()
        t = tuple(range(n))
        for i in range(n):
            lst.append(i)
            self.assertEqual(tuple(lst), t[:i+1])

    def test_delstuff(self):
        lst = self.type2test(list(range(n)))
        t = tuple(range(n))
        x = lst[4:258]
        self.assertEqual(tuple(x), tuple(t[4:258]))
        x.append(-1)
        self.assertEqual(tuple(x), tuple(t[4:258] + (-1,)))
        self.assertEqual(tuple(lst), t)
        lst[200] = 6
        self.assertEqual(tuple(x), tuple(t[4:258] + (-1,)))
        self.assertEqual(tuple(lst), tuple(t[0:200] + (6,) + t[201:]))
        del lst[200]
        self.assertEqual(tuple(lst), tuple(t[0:200] + t[201:]))

    def test_del1(self):
        lst2 = self.type2test(list(range(limit+1)))
        self.assertEqual(tuple(lst2), tuple(range(limit+1)))
        del lst2[1]
        del lst2[-1]
        self.assertEqual(tuple(lst2), (0,) + tuple(range(2,limit)))

    def test_insert_and_del(self):
        lst = self.type2test(list(range(n)))
        t = tuple(range(n))
        lst.insert(200, 0)
        self.assertEqual(tuple(lst), (t[0:200] + (0,) + t[200:]))
        del lst[200:]
        self.assertEqual(tuple(lst), tuple(range(200)))

    def test_mul3(self):
        lst = self.type2test(list(range(3)))
        self.assertEqual(tuple(lst*3), tuple(list(range(3))*3))

    def test_mul(self):
        x = self.type2test(list(range(limit**2)))
        for i in range(10):
            self.assertEqual(len(x*i), i*limit**2)

    def test_extendspam(self):
        a = self.type2test('spam')
        a.extend('eggs')
        self.assertEqual(list(a), list('spameggs'))

    def test_bigmul1(self):
        x = self.type2test([0])
        for i in list(range(290)) + [1000, 10000, 100000, 1000000, 10000000, 2**29]:
            self.assertEqual(len(x*i), i)

    def test_badinit(self):
        self.assertRaises(TypeError, self.type2test, 0, 0, 0)

    def test_copyself(self):
        x = self.type2test(list(range(n)))
        x[:] = x

    def test_nohash(self):
        x = self.type2test()
        d = {}
        self.assertRaises(TypeError, d.__setitem__, x, 5)

    def test_collapseboth(self):
        x = self.type2test(list(range(512)))
        del x[193:318]

    def test_collapseright(self):
        x = self.type2test(list(range(512)))
        del x[248:318]

    def test_badrepr(self):
        class BadExc(Exception):
            pass

        class BadRepr:
            def __repr__(self):
                raise BadExc

        x = self.type2test([BadRepr()])
        self.assertRaises(BadExc, repr, x)
        x = self.type2test(list(range(n)))
        x.append(BadRepr())
        self.assertRaises(BadExc, repr, x)

    def test_slice0(self):
        x = self.type2test(list(range(n)))
        x[slice(5,3,1)] = []
        self.assertEqual(x, list(range(n)))
        x = self.type2test(list(range(n)))
        self.assertRaises(ValueError, x.__setitem__, slice(5,3,1), [5,3,2])
        del x[slice(5,3,1)]
        self.assertEqual(x, list(range(n)))

    def test_badindex(self):
        x = self.type2test()
        self.assertRaises(TypeError, x.__setitem__, 's', 5)

    def test_comparelist(self):
        x = self.type2test(list(range(n)))
        y = list(range(n-1))
        self.assert_(not (x == y))
        self.assert_(x != y)
        self.assert_(not (x < y))
        self.assert_(not (x <= y))
        self.assert_(x > y)
        self.assert_(x >= y)

        y = list(range(n))
        self.assert_(x == y)
        self.assert_(y == x)

        y[100] = 6
        self.assert_(not (x == y))
        self.assert_(x != y)

    def test_compareblist(self):
        x = self.type2test(list(range(n)))
        y = self.type2test(list(range(n-1)))
        self.assert_(not (x == y))
        self.assert_(x != y)
        self.assert_(not (x < y))
        self.assert_(not (x <= y))
        self.assert_(x > y)
        self.assert_(x >= y)

        y[100] = 6
        self.assert_(not (x == y))
        self.assert_(x != y)

    def test_comparetuple(self):
        x = self.type2test(list(range(n)))
        y = tuple(range(n))
        self.assert_(x != y)

    def test_indexempty(self):
        x = self.type2test(list(range(10)))
        self.assertRaises(ValueError, x.index, 'spam')

    def test_indexargs(self):
        x = self.type2test(list(range(10)))
        self.assertEqual(x.index(5,1,-1), 5)
        self.assertRaises(ValueError, x.index, 5, -1, -9)
        self.assertRaises(ValueError, x.index, 8, 1, 4)
        self.assertRaises(ValueError, x.index, 0, 1, 4)

    def test_reversebig(self):
        x = self.type2test(list(range(n)))
        x.reverse()
        self.assertEqual(x, list(range(n-1,-1,-1)))

    def test_badconcat(self):
        x = self.type2test()
        y = 'foo'
        self.assertRaises(TypeError, operator.add, x, y)

    def test_bad_assign(self):
        x = self.type2test(list(range(n)))
        self.assertRaises(TypeError, x.__setitem__, slice(1,10,2), 5)

    def sort_evil(self, after):
        class EvilCompare:
            count = 0
            num_raises = 0
            def __init__(self, x):
                self.x = x
            def __lt__(self, other):
                EvilCompare.count += 1
                if EvilCompare.count > after:
                    EvilCompare.num_raises += 1
                    raise ValueError
                return self.x < other.x

        x = self.type2test(EvilCompare(x) for x in range(n))
        from random import shuffle
        shuffle(x)
        self.assertRaises(ValueError, x.sort)
        self.assertEqual(EvilCompare.num_raises, 1)
        x = [a.x for a in x]
        x.sort()
        self.assertEquals(x, list(range(n)))

    def test_sort_evil_small(self):
        self.sort_evil(limit * 5)

    def test_sort_evil_big(self):
        self.sort_evil(n + limit)

    def test_big_extend(self):
        x = self.type2test([1])
        x.extend(range(n))
        self.assertEqual(tuple(x), (1,) + tuple(range(n)))

    def test_big_getslice(self):
        x = self.type2test([0]) * 65536
        self.assertEqual(len(x[256:512]), 256)

    def test_modify_original(self):
        x = self.type2test(list(range(1024)))
        y = x[:]
        x[5] = 'z'
        self.assertEqual(tuple(y), tuple(range(1024)))
        self.assertEqual(x[5], 'z')
        self.assertEqual(tuple(x[:5]), tuple(range(5)))
        self.assertEqual(tuple(x[6:]), tuple(range(6, 1024)))

    def test_modify_copy(self):
        x = self.type2test(list(range(1024)))
        y = x[:]
        y[5] = 'z'
        self.assertEqual(tuple(x), tuple(range(1024)))
        self.assertEqual(y[5], 'z')
        self.assertEqual(tuple(y[:5]), tuple(range(5)))
        self.assertEqual(tuple(y[6:]), tuple(range(6, 1024)))

    def test_bigsort(self):
        x = self.type2test(list(range(100000)))
        x.sort()

    def test_sort_twice(self):
        y = blist.blist(list(range(limit+1)))
        for i in range(2):
            x = blist.blist(y)
            x.sort()
            self.assertEqual(tuple(x), tuple(range(limit+1)))

    def test_LIFO(self):
        x = blist.blist()
        for i in range(1000):
            x.append(i)
        for j in range(1000-1,-1,-1):
            self.assertEqual(x.pop(), j)

    def pickle_test(self, pickler, x):
        y = pickler.dumps(x)
        z = pickler.loads(y)
        self.assertEqual(x, z)
        self.assertEqual(repr(x), repr(z))

    def pickle_tests(self, pickler):
        self.pickle_test(pickler, blist.blist())
        self.pickle_test(pickler, blist.blist(list(range(limit))))
        self.pickle_test(pickler, blist.blist(list(range(limit+1))))
        self.pickle_test(pickler, blist.blist(list(range(n))))

        x = blist.blist([0])
        x *= n
        self.pickle_test(pickler, x)
        y = blist.blist(x)
        y[5] = 'x'
        self.pickle_test(pickler, x)
        self.pickle_test(pickler, y)

    def test_pickle(self):
        self.pickle_tests(pickle)

    def test_types(self):
        type(blist.blist())
        type(iter(blist.blist()))
        type(iter(reversed(blist.blist())))

    def test_iterlen_empty(self):
        it = iter(blist.blist())
        if hasattr(it, '__next__'): # pragma: no cover
            self.assertRaises(StopIteration, it.__next__)
        else: # pragma: no cover
            self.assertRaises(StopIteration, it.next)
        self.assertEqual(it.__length_hint__(), 0)

    def test_sort_floats(self):
        x = blist.blist([0.1, 0.2, 0.3])
        x.sort()

tests = [BListTest,
         sortedlist_tests.SortedListTest,
         sortedlist_tests.WeakSortedListTest,
         sortedlist_tests.SortedSetTest,
         sortedlist_tests.WeakSortedSetTest,
         btuple_tests.bTupleTest,
         sorteddict_tests.sorteddict_test
         ]
tests += test_set.test_classes

def test_suite():
    suite = unittest.TestSuite()
    for test in tests:
        suite.addTest(unittest.TestLoader().loadTestsFromTestCase(test))
    return suite

def test_main(verbose=None):
    test_support.run_unittest(*tests)

if __name__ == "__main__":
    test_main(verbose=True)
